'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

require('./dropdown.scss');

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _button = require('../button/button.js');

var _button2 = _interopRequireDefault(_button);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* eslint-disable react/no-set-state */


var optionHeight = 48;

/**
 * Dropdown
 * @param   {Object} props     - React props
 * @returns    {JSX} template  - Component template
 */

var Dropdown = function (_Component) {
  _inherits(Dropdown, _Component);

  /**
   * constructor
   * @param {Object} props - React props
   */
  function Dropdown(props) {
    _classCallCheck(this, Dropdown);

    var _this = _possibleConstructorReturn(this, (Dropdown.__proto__ || Object.getPrototypeOf(Dropdown)).call(this, props));

    _this._options = [];

    _this.state = { isOpen: Boolean(props.isOpen) };
    _this.close = _this.close.bind(_this);
    _this.select = _this.select.bind(_this);
    _this.setFocusedOption = _this.setFocusedOption.bind(_this);
    _this.toggleOpen = _this.toggleOpen.bind(_this);
    _this.handleSelectBlur = _this.handleSelectBlur.bind(_this);
    _this.handleSelectKeyDown = _this.handleSelectKeyDown.bind(_this);
    _this.handleSelectKeyUp = _this.handleSelectKeyUp.bind(_this);
    _this.handleTriggerKeyUp = _this.handleTriggerKeyUp.bind(_this);
    _this.handleOptionFocused = _this.handleOptionFocused.bind(_this);
    _this.handleOptionSelected = _this.handleOptionSelected.bind(_this);
    _this.handleOptionsMouseDown = _this.handleOptionsMouseDown.bind(_this);
    return _this;
  }

  /**
   * When component unmounts remove click outside listener
   */


  _createClass(Dropdown, [{
    key: 'componentWillUnmount',
    value: function componentWillUnmount() {
      document.removeEventListener('click', this.close);
    }

    /**
     * Closes the dropdown element
     */

  }, {
    key: 'close',
    value: function close() {
      this.setState({ isOpen: false, optionFocused: null });
      document.removeEventListener('click', this.close);
    }

    /**
     * Selects an option and closes
     * @param {string} value - Selected value
     */

  }, {
    key: 'select',
    value: function select(value) {
      if (this.props.onSelect) {
        this.props.onSelect(value);
      }
      this.close();
    }

    /**
     * Sets focus and scrolls container to an option element
     * @param {number} index - Index of the element to set focused
     */

  }, {
    key: 'setFocusedOption',
    value: function setFocusedOption(index) {
      var _props = this.props,
          size = _props.size,
          options = _props.options,
          elem = this._options[index];

      // istanbul ignore next
      // When limited size and capped scroll to visible is needed

      if (size && options && size < options.length) {
        if (elem.offsetTop + elem.offsetHeight > this._list.offsetHeight) {
          this._list.scrollTop = elem.offsetTop + elem.offsetHeight - this._list.offsetHeight;
        } else if (elem.offsetTop < this._list.scrollTop) {
          this._list.scrollTop = elem.offsetTop;
        }
      }

      this.setState({ optionFocused: index });
    }

    /**
     * Opens or closes the dropdown element
     */

  }, {
    key: 'toggleOpen',
    value: function toggleOpen() {
      this.setState({ isOpen: !this.state.isOpen, optionFocused: null });
      if (!this.state.isOpen) {
        document.addEventListener('click', this.close);
      }
    }

    /**
     * When trigger has enter or space pressed open dropdown
     * @param {object} event - DOM event object
     */

  }, {
    key: 'handleTriggerKeyUp',
    value: function handleTriggerKeyUp(event) {
      var code = event.keyCode || event.which;
      if (code === 13 || code === 32) {
        event.preventDefault();
        this.toggleOpen();
      }
    }

    /**
     * Close dropdown on blur
     */

  }, {
    key: 'handleSelectBlur',
    value: function handleSelectBlur() {
      this.close();
    }

    /**
     * Avoid blur to be fired before click an option
     * @param {object} event - DOM event object
     */

  }, {
    key: 'handleOptionsMouseDown',
    value: function handleOptionsMouseDown(event) {
      event.preventDefault();
    }

    /**
     * Disable scroll on page
     * @param {object} event - DOM event object
     */

  }, {
    key: 'handleSelectKeyDown',
    value: function handleSelectKeyDown(event) {
      // istanbul ignore next
      var code = event.keyCode || event.which;
      if (code === 38 || code === 40 || code === 13 || code === 32) {
        event.preventDefault();
        event.stopPropagation();
      }
    }

    /**
     * Handle keys for
     *  - cycling through options and select one
     *  - close when escape
     *  - select on enter
     * @param {object} event - DOM event object
     */

  }, {
    key: 'handleSelectKeyUp',
    value: function handleSelectKeyUp(event) {
      var options = this.props.options,
          _state = this.state,
          isOpen = _state.isOpen,
          optionFocused = _state.optionFocused;
      // istanbul ignore next
      var code = event.keyCode || event.which;
      switch (code) {
        case 38:
          // up arrow
          event.preventDefault();
          if (optionFocused > 0) {
            this.setFocusedOption(optionFocused - 1);
          }
          break;
        case 40:
          // down arrow
          event.preventDefault();
          if (optionFocused === null) {
            this.setFocusedOption(optionFocused - 0);
          } else if (optionFocused < options.length - 1) {
            this.setFocusedOption(optionFocused + 1);
          }
          break;
        case 13: // enter
        case 32:
          // space
          event.preventDefault();
          if (optionFocused !== null && isOpen) {
            this.select(options[optionFocused]);
          }
          break;
        case 27:
          // esc
          event.preventDefault();
          this.close();
          break;
        default:
          break;
      }
    }

    /**
     * Handler when an option is focused by:
     *  - hover
     *  - focus
     * @param {string} index - Focused option index
     * @returns {function}   - Event handler function
     */

  }, {
    key: 'handleOptionFocused',
    value: function handleOptionFocused(index) {
      var _this2 = this;

      return function () {
        _this2.setFocusedOption(index);
      };
    }

    /**
     * Handler when an option is selected by:
     *  - click
     *  - press enter or space
     * @param {string} value - Selected option value
     * @returns {function}   - Event handler function
     */

  }, {
    key: 'handleOptionSelected',
    value: function handleOptionSelected(value) {
      var _this3 = this;

      return function (event) {
        var code = event.key || event.keyCode || event.which;
        if (code === 13 || code === 32 || !code) {
          _this3.select(value);
        }
      };
    }

    /**
     * Render method
     * @returns {JSX} - Component template
     */

  }, {
    key: 'render',
    value: function render() {
      var _this4 = this;

      var isOpen = this.state.isOpen,
          _props2 = this.props,
          caption = _props2.caption,
          children = _props2.children,
          options = _props2.options,
          size = _props2.size;

      var listStyle = {};
      var trigger = (0, _react.cloneElement)(children || _react2.default.createElement(_button2.default, { arrow: this.state.isOpen ? 'up' : 'down', caption: caption }), _extends({}, children ? children.props : {}, {
        className: (0, _classnames2.default)('trigger', children ? children.props.className : ''),
        onClick: this.toggleOpen,
        onKeyUp: this.handleTriggerKeyUp,
        tabIndex: '0'
      }));

      if (size && options && size < options.length) {
        listStyle.maxHeight = optionHeight * (size + 0.5) + 'px';
        listStyle.overflowY = 'scroll';
      }

      return _react2.default.createElement(
        'div',
        {
          className: (0, _classnames2.default)('dropdown', { 'is-open': isOpen }),
          onBlur: this.handleSelectBlur,
          onKeyDown: this.handleSelectKeyDown,
          onKeyUp: this.handleSelectKeyUp,
          tabIndex: '-1'
        },
        trigger,
        this.state.isOpen && _react2.default.createElement(
          'ul',
          {
            className: 'options',
            onMouseDown: this.handleOptionsMouseDown,
            ref: function ref(_ref2) {
              _this4._list = _ref2;
            },
            role: 'listbox',
            style: listStyle
          },
          options && options.map(function (option, index) {
            return _react2.default.createElement(
              'li',
              {
                className: (0, _classnames2.default)('option', { focus: _this4.state.optionFocused === index }),
                key: index,
                onClick: _this4.handleOptionSelected(option.value || option),
                onFocus: _this4.handleOptionFocused(index),
                onKeyUp: _this4.handleOptionSelected(option.value || option),
                onMouseOver: _this4.handleOptionFocused(index),
                ref: function ref(_ref) {
                  _this4._options[index] = _ref;
                },
                role: 'option',
                tabIndex: '-1'
              },
              option.label || option
            );
          })
        )
      );
    }
  }]);

  return Dropdown;
}(_react.Component);

exports.default = Dropdown;


Dropdown.propTypes = {
  caption: _propTypes2.default.string,
  children: _propTypes2.default.element,
  isOpen: _propTypes2.default.bool,
  onSelect: _propTypes2.default.func,
  options: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.shape({
    value: _propTypes2.default.string,
    label: _propTypes2.default.string
  })])),
  size: _propTypes2.default.number
};