'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

require('./field.scss');

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _classnames = require('classnames');

var _classnames2 = _interopRequireDefault(_classnames);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _select = require('./select.js');

var _select2 = _interopRequireDefault(_select);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* eslint-disable react/no-set-state */


/**
 * Field
 * Wraps the label and error of any form field
 * @param {Object} props    - React props
 * @returns  {JSX} template - Component template
 */
var Field = function (_Component) {
  _inherits(Field, _Component);

  /**
   * constructor
   * @param {Object} props - React props
   */
  function Field(props) {
    _classCallCheck(this, Field);

    var _this = _possibleConstructorReturn(this, (Field.__proto__ || Object.getPrototypeOf(Field)).call(this, props));

    _this.state = { errorMessage: '', invalid: false };
    _this.handleChange = _this.handleChange.bind(_this);
    _this.handleInvalid = _this.handleInvalid.bind(_this);
    return _this;
  }

  /**
   * Component mounted on DOM
   */


  _createClass(Field, [{
    key: 'componentDidMount',
    value: function componentDidMount() {
      var elem = this._formElem;
      if (this.props.validate && elem.checkValidity) {
        elem.checkValidity();
      }
    }

    /**
     * When Change response with just value
     * @param {object} event - DOM event object
     */

  }, {
    key: 'handleChange',
    value: function handleChange(event) {
      if (this.state.invalid || this.props.validate) {
        this.validate(event.target);
      }
      // istanbul ignore else
      if (this.props.onChange) {
        this.props.onChange(event.target.value);
      }
    }

    /**
     * When Invalid value stop default browser validation popups
     * and do custom field validation
     * @param {object} event - DOM event object
     */

  }, {
    key: 'handleInvalid',
    value: function handleInvalid(event) {
      event.preventDefault();
      this.setState({ invalid: true });
      this.validate(event.target);
    }

    /**
     * Analyze validity and set error message
     * @param {object} elem - Form element to be validated
     */

  }, {
    key: 'validate',
    value: function validate(elem) {
      var _props = this.props,
          errorMessages = _props.errorMessages,
          type = _props.type;

      if (elem.validity && !elem.validity.valid) {
        if (elem.validity.valueMissing) {
          this.setState({
            errorMessage: errorMessages.valueMissing || 'This is a required field.',
            invalid: true
          });
        } else if (elem.validity.typeMismatch) {
          this.setState({
            errorMessage: errorMessages.typeMismatch || 'This value is not an ' + type + '.',
            invalid: true
          });
        } else {
          this.setState({
            errorMessage: errorMessages.patternMismatch || 'This value is invalid.',
            invalid: true
          });
        }
      } else {
        this.setState({ errorMessage: '', invalid: false });
      }
    }

    /**
     * Render method
     * @returns {JSX} - Component template
     */

  }, {
    key: 'render',
    value: function render() {
      var _this2 = this;

      var _props2 = this.props,
          className = _props2.className,
          component = _props2.component,
          disabled = _props2.disabled,
          id = _props2.id,
          inline = _props2.inline,
          label = _props2.label,
          readOnly = _props2.readOnly,
          required = _props2.required,
          errorMessages = _props2.errorMessages,
          multiLine = _props2.multiLine,
          resize = _props2.resize,
          validate = _props2.validate,
          other = _objectWithoutProperties(_props2, ['className', 'component', 'disabled', 'id', 'inline', 'label', 'readOnly', 'required', 'errorMessages', 'multiLine', 'resize', 'validate']);

      var _state = this.state,
          errorMessage = _state.errorMessage,
          invalid = _state.invalid;

      var FormElement = void 0;
      var Element = inline ? 'span' : 'div',
          fieldCSS = (0, _classnames2.default)('field', { disabled: disabled, inline: inline, invalid: invalid, readonly: readOnly }, className),
          ariaAttrs = !invalid ? {} : { 'aria-invalid': invalid, 'aria-describedby': id + '-error' },

      // eslint-disable-next-line no-nested-ternary
      elementStyle = { resize: !resize ? 'none' : resize === true ? 'both' : resize };

      FormElement = component === 'select' ? _select2.default : component;

      if (multiLine && component === 'input') {
        FormElement = 'textarea';
      }

      return _react2.default.createElement(
        Element,
        { className: fieldCSS },
        label && _react2.default.createElement(
          'label',
          { className: 'field-label', htmlFor: id },
          label,
          required && _react2.default.createElement('span', {
            'aria-label': 'Required',
            className: 'icon icon-required',
            title: 'Required'
          })
        ),
        _react2.default.createElement(FormElement, _extends({}, other, ariaAttrs, {
          className: 'field-element',
          disabled: disabled,
          id: id,
          onChange: this.handleChange,
          onInvalid: this.handleInvalid,
          readOnly: readOnly,
          ref: function ref(_ref) {
            _this2._formElem = _ref;
          },
          required: required,
          style: elementStyle
        })),
        errorMessage && _react2.default.createElement(
          Element,
          { className: 'field-error', id: id + '-error' },
          errorMessage
        )
      );
    }
  }]);

  return Field;
}(_react.Component);

exports.default = Field;


Field.propTypes = {
  autoComplete: _propTypes2.default.bool,
  autoFocus: _propTypes2.default.bool,
  className: _propTypes2.default.string,
  cols: _propTypes2.default.number,
  component: _propTypes2.default.oneOf(['input', 'select', 'textarea']),
  defaultValue: _propTypes2.default.string,
  disabled: _propTypes2.default.bool,
  errorMessages: _propTypes2.default.shape({
    patternMismatch: _propTypes2.default.string,
    typeMismatch: _propTypes2.default.string,
    valueMissing: _propTypes2.default.string
  }),
  form: _propTypes2.default.string,
  formaction: _propTypes2.default.string,
  formenctype: _propTypes2.default.string,
  formmethod: _propTypes2.default.oneOf(['get', 'post']),
  formnovalidate: _propTypes2.default.bool,
  formtarget: _propTypes2.default.string,
  id: _propTypes2.default.string,
  inline: _propTypes2.default.bool,
  label: _propTypes2.default.string,
  max: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]),
  maxLength: _propTypes2.default.number,
  min: _propTypes2.default.oneOfType([_propTypes2.default.number, _propTypes2.default.string]),
  minLength: _propTypes2.default.number,
  multiLine: _propTypes2.default.bool,
  name: _propTypes2.default.string,
  onBlur: _propTypes2.default.func,
  onChange: _propTypes2.default.func,
  onFocus: _propTypes2.default.func,
  onInvalid: _propTypes2.default.func,
  pattern: _propTypes2.default.string,
  placeholder: _propTypes2.default.string,
  readOnly: _propTypes2.default.bool,
  required: _propTypes2.default.bool,
  resize: _propTypes2.default.oneOfType([_propTypes2.default.bool, _propTypes2.default.oneOf(['none', 'both', 'horizontal', 'vertical', 'inherit'])]),
  rows: _propTypes2.default.number,
  size: _propTypes2.default.number,
  spellCheck: _propTypes2.default.bool,
  step: _propTypes2.default.number,
  tabIndex: _propTypes2.default.number,
  type: _propTypes2.default.oneOf(['text', 'email', 'number', 'password', 'hidden', 'tel', 'url', 'color', 'search', 'range', 'date', 'datetime-local', 'week', 'month']),
  validate: _propTypes2.default.bool,
  wrap: _propTypes2.default.oneOf(['hard', 'soft'])
};

Field.defaultProps = { component: 'input', errorMessages: {}, type: 'text' };